@@ -41,6 +41,9 @@ def unpack_infer(stmt, context=None):
     """
     if isinstance(stmt, (List, Tuple)):
         for elt in stmt.elts:
+            if elt is util.YES:
+                yield elt
+                continue
             for inferred_elt in unpack_infer(elt, context):
                 yield inferred_elt
         return
@@ -111,17 +114,6 @@ def are_exclusive(stmt1, stmt2, exceptions=None):
     return False
 
 
-def _container_getitem(instance, elts, index):
-    """Get a slice or an item, using the given *index*, for the given sequence."""
-    if isinstance(index, slice):
-        new_cls = instance.__class__()
-        new_cls.elts = elts[index]
-        new_cls.parent = instance.parent
-        return new_cls
-    else:
-        return elts[index]
-
-
 @six.add_metaclass(abc.ABCMeta)
 class _BaseContainer(mixins.ParentAssignTypeMixin,
                      bases.NodeNG,
@@ -130,22 +122,11 @@ class _BaseContainer(mixins.ParentAssignTypeMixin,
 
     _astroid_fields = ('elts',)
 
-    def __init__(self, lineno=None, col_offset=None, parent=None):
-        self.elts = []
-        super(_BaseContainer, self).__init__(lineno, col_offset, parent)
-
-    def postinit(self, elts):
-        self.elts = elts
-
-    @classmethod
-    def from_constants(cls, elts=None):
-        # pylint: disable=abstract-class-instantiated; False positive on Pylint #627.
-        node = cls()
+    def __init__(self, elts=None):
         if elts is None:
-            node.elts = []
+            self.elts = []
         else:
-            node.elts = [const_factory(e) for e in elts]
-        return node
+            self.elts = [const_factory(e) for e in elts]
 
     def itered(self):
         return self.elts
@@ -236,7 +217,7 @@ class LookupMixIn(object):
             if mylineno > 0 and stmt.fromlineno > mylineno:
                 break
             assert hasattr(node, 'assign_type'), (node, node.scope(),
-                                                  node.scope().locals)
+                                                  node.scope()._locals)
             assign_type = node.assign_type()
 
             if node.has_base(self):
@@ -308,30 +289,15 @@ class LookupMixIn(object):
 # Name classes
 
 class AssignName(LookupMixIn, mixins.ParentAssignTypeMixin, bases.NodeNG):
-    """class representing an AssignName node"""
-    _other_fields = ('name',)
-
-    def __init__(self, name=None, lineno=None, col_offset=None, parent=None):
-        self.name = name
-        super(AssignName, self).__init__(lineno, col_offset, parent)
+    """class representing an AssName node"""
 
 
 class DelName(LookupMixIn, mixins.ParentAssignTypeMixin, bases.NodeNG):
     """class representing a DelName node"""
-    _other_fields = ('name',)
-
-    def __init__(self, name=None, lineno=None, col_offset=None, parent=None):
-        self.name = name
-        super(DelName, self).__init__(lineno, col_offset, parent)
 
 
 class Name(LookupMixIn, bases.NodeNG):
     """class representing a Name node"""
-    _other_fields = ('name',)
-
-    def __init__(self, name=None, lineno=None, col_offset=None, parent=None):
-        self.name = name
-        super(Name, self).__init__(lineno, col_offset, parent)
 
 
 class Arguments(mixins.AssignTypeMixin, bases.NodeNG):
@@ -349,33 +315,21 @@ class Arguments(mixins.AssignTypeMixin, bases.NodeNG):
         #    annotation, its value will be None.
 
         _astroid_fields = ('args', 'defaults', 'kwonlyargs',
-                           'kw_defaults', 'annotations', 'varargannotation',
-                           'kwargannotation')
+                           'kw_defaults', 'annotations',
+                           'varargannotation', 'kwargannotation')
+        annotations = None
         varargannotation = None
         kwargannotation = None
     else:
         _astroid_fields = ('args', 'defaults', 'kwonlyargs', 'kw_defaults')
-        _other_fields = ('vararg', 'kwarg')
+    args = None
+    defaults = None
+    kwonlyargs = None
+    kw_defaults = None
 
-    def __init__(self, vararg=None, kwarg=None, parent=None):
+    def __init__(self, vararg=None, kwarg=None):
         self.vararg = vararg
         self.kwarg = kwarg
-        self.parent = parent
-        self.args = []
-        self.defaults = []
-        self.kwonlyargs = []
-        self.kw_defaults = []
-        self.annotations = []
-
-    def postinit(self, args, defaults, kwonlyargs, kw_defaults,
-                 annotations, varargannotation=None, kwargannotation=None):
-        self.args = args
-        self.defaults = defaults
-        self.kwonlyargs = kwonlyargs
-        self.kw_defaults = kw_defaults
-        self.annotations = annotations
-        self.varargannotation = varargannotation
-        self.kwargannotation = kwargannotation
 
     def _infer_name(self, frame, name):
         if self.parent is frame:
@@ -480,128 +434,42 @@ def _format_args(args, defaults=None, annotations=None):
 class AssignAttr(mixins.ParentAssignTypeMixin, bases.NodeNG):
     """class representing an AssignAttr node"""
     _astroid_fields = ('expr',)
-    _other_fields = ('attrname',)
     expr = None
 
-    def __init__(self, attrname=None, lineno=None, col_offset=None, parent=None):
-        self.attrname = attrname
-        super(AssignAttr, self).__init__(lineno, col_offset, parent)
-
-    def postinit(self, expr=None):
-        self.expr = expr
-
-
 class Assert(bases.Statement):
     """class representing an Assert node"""
     _astroid_fields = ('test', 'fail',)
     test = None
     fail = None
 
-    def postinit(self, test=None, fail=None):
-        self.fail = fail
-        self.test = test
-
-
-class Assign(mixins.AssignTypeMixin, bases.Statement):
+class Assign(bases.Statement, mixins.AssignTypeMixin):
     """class representing an Assign node"""
     _astroid_fields = ('targets', 'value',)
     targets = None
     value = None
 
-    def postinit(self, targets=None, value=None):
-        self.targets = targets
-        self.value = value
-
-
-class AugAssign(mixins.AssignTypeMixin, bases.Statement):
+class AugAssign(bases.Statement, mixins.AssignTypeMixin):
     """class representing an AugAssign node"""
-    _astroid_fields = ('target', 'value')
-    _other_fields = ('op',)
+    _astroid_fields = ('target', 'value',)
     target = None
     value = None
 
-    def __init__(self, op=None, lineno=None, col_offset=None, parent=None):
-        self.op = op
-        super(AugAssign, self).__init__(lineno, col_offset, parent)
-
-    def postinit(self, target=None, value=None):
-        self.target = target
-        self.value = value
-
-    # This is set by inference.py
-    def _infer_augassign(self, context=None):
-        raise NotImplementedError
-
-    def type_errors(self, context=None):
-        """Return a list of TypeErrors which can occur during inference.
-
-        Each TypeError is represented by a :class:`BinaryOperationError`,
-        which holds the original exception.
-        """
-        try:
-            results = self._infer_augassign(context=context)
-            return [result for result in results
-                    if isinstance(result, exceptions.BinaryOperationError)]
-        except exceptions.InferenceError:
-            return []
-
-
 class Repr(bases.NodeNG):
-    """class representing a Repr node"""
+    """class representing a Backquote node"""
     _astroid_fields = ('value',)
     value = None
 
-    def postinit(self, value=None):
-        self.value = value
-
-
 class BinOp(bases.NodeNG):
     """class representing a BinOp node"""
-    _astroid_fields = ('left', 'right')
-    _other_fields = ('op',)
+    _astroid_fields = ('left', 'right',)
     left = None
     right = None
 
-    def __init__(self, op=None, lineno=None, col_offset=None, parent=None):
-        self.op = op
-        super(BinOp, self).__init__(lineno, col_offset, parent)
-
-    def postinit(self, left=None, right=None):
-        self.left = left
-        self.right = right
-
-    # This is set by inference.py
-    def _infer_binop(self, context=None):
-        raise NotImplementedError
-
-    def type_errors(self, context=None):
-        """Return a list of TypeErrors which can occur during inference.
-
-        Each TypeError is represented by a :class:`BinaryOperationError`,
-        which holds the original exception.
-        """
-        try:
-            results = self._infer_binop(context=context)
-            return [result for result in results
-                    if isinstance(result, exceptions.BinaryOperationError)]
-        except exceptions.InferenceError:
-            return []
-
-
 class BoolOp(bases.NodeNG):
     """class representing a BoolOp node"""
     _astroid_fields = ('values',)
-    _other_fields = ('op',)
     values = None
 
-    def __init__(self, op=None, lineno=None, col_offset=None, parent=None):
-        self.op = op
-        super(BoolOp, self).__init__(lineno, col_offset, parent)
-
-    def postinit(self, values=None):
-        self.values = values
-
-
 class Break(bases.Statement):
     """class representing a Break node"""
 
@@ -613,11 +481,6 @@ class Call(bases.NodeNG):
     args = None
     keywords = None
 
-    def postinit(self, func=None, args=None, keywords=None):
-        self.func = func
-        self.args = args
-        self.keywords = keywords
-
     @property
     def starargs(self):
         args = self.args or []
@@ -628,17 +491,12 @@ class Call(bases.NodeNG):
         keywords = self.keywords or []
         return [keyword for keyword in keywords if keyword.arg is None]
 
-
 class Compare(bases.NodeNG):
     """class representing a Compare node"""
     _astroid_fields = ('left', 'ops',)
     left = None
     ops = None
 
-    def postinit(self, left=None, ops=None):
-        self.left = left
-        self.ops = ops
-
     def get_children(self):
         """override get_children for tuple fields"""
         yield self.left
@@ -659,14 +517,6 @@ class Comprehension(bases.NodeNG):
     iter = None
     ifs = None
 
-    def __init__(self, parent=None):
-        self.parent = parent
-
-    def postinit(self, target=None, iter=None, ifs=None):
-        self.target = target
-        self.iter = iter
-        self.ifs = ifs
-
     optional_assign = True
     def assign_type(self):
         return self
@@ -695,11 +545,9 @@ class Comprehension(bases.NodeNG):
 
 class Const(bases.NodeNG, bases.Instance):
     """represent a constant node like num, str, bool, None, bytes"""
-    _other_fields = ('value',)
 
-    def __init__(self, value, lineno=None, col_offset=None, parent=None):
+    def __init__(self, value=None):
         self.value = value
-        super(Const, self).__init__(lineno, col_offset, parent)
 
     def getitem(self, index, context=None):
         if isinstance(self.value, six.string_types):
@@ -722,9 +570,6 @@ class Const(bases.NodeNG, bases.Instance):
     def pytype(self):
         return self._proxied.qname()
 
-    def bool_value(self):
-        return bool(self.value)
-
 
 class Continue(bases.Statement):
     """class representing a Continue node"""
@@ -735,7 +580,7 @@ class Decorators(bases.NodeNG):
     _astroid_fields = ('nodes',)
     nodes = None
 
-    def postinit(self, nodes):
+    def __init__(self, nodes=None):
         self.nodes = nodes
 
     def scope(self):
@@ -746,15 +591,8 @@ class Decorators(bases.NodeNG):
 class DelAttr(mixins.ParentAssignTypeMixin, bases.NodeNG):
     """class representing a DelAttr node"""
     _astroid_fields = ('expr',)
-    _other_fields = ('attrname',)
     expr = None
 
-    def __init__(self, attrname=None, lineno=None, col_offset=None, parent=None):
-        self.attrname = attrname
-        super(DelAttr, self).__init__(lineno, col_offset, parent)
-
-    def postinit(self, expr=None):
-        self.expr = expr
 
 
 class Delete(mixins.AssignTypeMixin, bases.Statement):
@@ -762,30 +600,17 @@ class Delete(mixins.AssignTypeMixin, bases.Statement):
     _astroid_fields = ('targets',)
     targets = None
 
-    def postinit(self, targets=None):
-        self.targets = targets
-
 
 class Dict(bases.NodeNG, bases.Instance):
     """class representing a Dict node"""
     _astroid_fields = ('items',)
 
-    def __init__(self, lineno=None, col_offset=None, parent=None):
-        self.items = []
-        super(Dict, self).__init__(lineno, col_offset, parent)
-
-    def postinit(self, items):
-        self.items = items
-
-    @classmethod
-    def from_constants(cls, items=None):
-        node = cls()
+    def __init__(self, items=None):
         if items is None:
-            node.items = []
+            self.items = []
         else:
-            node.items = [(const_factory(k), const_factory(v))
-                          for k, v in items.items()]
-        return node
+            self.items = [(const_factory(k), const_factory(v))
+                          for k, v in list(items.items())]
 
     def pytype(self):
         return '%s.dict' % BUILTINS
@@ -808,6 +633,12 @@ class Dict(bases.NodeNG, bases.Instance):
 
     def getitem(self, lookup_key, context=None):
         for key, value in self.items:
+            # TODO(cpopa): no support for overriding yet, {1:2, **{1: 3}}.
+            if isinstance(key, DictUnpack):
+                try:
+                    return value.getitem(lookup_key, context)
+                except IndexError:
+                    continue
             for inferredkey in key.infer(context):
                 if inferredkey is util.YES:
                     continue
@@ -818,25 +649,16 @@ class Dict(bases.NodeNG, bases.Instance):
         # IndexError. Let's leave it like that for now.
         raise IndexError(lookup_key)
 
-    def bool_value(self):
-        return bool(self.items)
-
 
 class Expr(bases.Statement):
     """class representing a Expr node"""
     _astroid_fields = ('value',)
     value = None
 
-    def postinit(self, value=None):
-        self.value = value
-
 
 class Ellipsis(bases.NodeNG): # pylint: disable=redefined-builtin
     """class representing an Ellipsis node"""
 
-    def bool_value(self):
-        return True
-
 
 class EmptyNode(bases.NodeNG):
     """class representing an EmptyNode node"""
@@ -849,11 +671,6 @@ class ExceptHandler(mixins.AssignTypeMixin, bases.Statement):
     name = None
     body = None
 
-    def postinit(self, type=None, name=None, body=None):
-        self.type = type
-        self.name = name
-        self.body = body
-
     @decorators.cachedproperty
     def blockstart_tolineno(self):
         if self.name:
@@ -878,21 +695,12 @@ class Exec(bases.Statement):
     globals = None
     locals = None
 
-    def postinit(self, expr=None, globals=None, locals=None):
-        self.expr = expr
-        self.globals = globals
-        self.locals = locals
-
 
 class ExtSlice(bases.NodeNG):
     """class representing an ExtSlice node"""
     _astroid_fields = ('dims',)
     dims = None
 
-    def postinit(self, dims=None):
-        self.dims = dims
-
-
 class For(mixins.BlockRangeMixIn, mixins.AssignTypeMixin, bases.Statement):
     """class representing a For node"""
     _astroid_fields = ('target', 'iter', 'body', 'orelse',)
@@ -901,12 +709,6 @@ class For(mixins.BlockRangeMixIn, mixins.AssignTypeMixin, bases.Statement):
     body = None
     orelse = None
 
-    def postinit(self, target=None, iter=None, body=None, orelse=None):
-        self.target = target
-        self.iter = iter
-        self.body = body
-        self.orelse = orelse
-
     optional_assign = True
     @decorators.cachedproperty
     def blockstart_tolineno(self):
@@ -928,38 +730,24 @@ class Await(bases.NodeNG):
 
 
 class ImportFrom(mixins.ImportFromMixin, bases.Statement):
-    """class representing a ImportFrom node"""
-    _other_fields = ('modname', 'names', 'level')
+    """class representing a From node"""
 
-    def __init__(self, fromname, names, level=0, lineno=None,
-                 col_offset=None, parent=None):
+    def __init__(self, fromname, names, level=0):
         self.modname = fromname
         self.names = names
         self.level = level
-        super(ImportFrom, self).__init__(lineno, col_offset, parent)
-
 
 class Attribute(bases.NodeNG):
     """class representing a Attribute node"""
     _astroid_fields = ('expr',)
-    _other_fields = ('attrname',)
     expr = None
 
-    def __init__(self, attrname=None, lineno=None, col_offset=None, parent=None):
-        self.attrname = attrname
-        super(Attribute, self).__init__(lineno, col_offset, parent)
-
-    def postinit(self, expr=None):
-        self.expr = expr
-
 
 class Global(bases.Statement):
     """class representing a Global node"""
-    _other_fields = ('names',)
 
-    def __init__(self, names, lineno=None, col_offset=None, parent=None):
+    def __init__(self, names):
         self.names = names
-        super(Global, self).__init__(lineno, col_offset, parent)
 
     def _infer_name(self, frame, name):
         return name
@@ -972,11 +760,6 @@ class If(mixins.BlockRangeMixIn, bases.Statement):
     body = None
     orelse = None
 
-    def postinit(self, test=None, body=None, orelse=None):
-        self.test = test
-        self.body = body
-        self.orelse = orelse
-
     @decorators.cachedproperty
     def blockstart_tolineno(self):
         return self.test.tolineno
@@ -998,19 +781,9 @@ class IfExp(bases.NodeNG):
     body = None
     orelse = None
 
-    def postinit(self, test=None, body=None, orelse=None):
-        self.test = test
-        self.body = body
-        self.orelse = orelse
-
 
 class Import(mixins.ImportFromMixin, bases.Statement):
     """class representing an Import node"""
-    _other_fields = ('names',)
-
-    def __init__(self, names=None, lineno=None, col_offset=None, parent=None):
-        self.names = names
-        super(Import, self).__init__(lineno, col_offset, parent)
 
 
 class Index(bases.NodeNG):
@@ -1018,23 +791,12 @@ class Index(bases.NodeNG):
     _astroid_fields = ('value',)
     value = None
 
-    def postinit(self, value=None):
-        self.value = value
-
 
 class Keyword(bases.NodeNG):
     """class representing a Keyword node"""
     _astroid_fields = ('value',)
-    _other_fields = ('arg',)
     value = None
 
-    def __init__(self, arg=None, lineno=None, col_offset=None, parent=None):
-        self.arg = arg
-        super(Keyword, self).__init__(lineno, col_offset, parent)
-
-    def postinit(self, value=None):
-        self.value = value
-
 
 class List(_BaseContainer):
     """class representing a List node"""
@@ -1043,16 +805,14 @@ class List(_BaseContainer):
         return '%s.list' % BUILTINS
 
     def getitem(self, index, context=None):
-        return _container_getitem(self, self.elts, index)
+        return self.elts[index]
 
 
 class Nonlocal(bases.Statement):
     """class representing a Nonlocal node"""
-    _other_fields = ('names',)
 
-    def __init__(self, names, lineno=None, col_offset=None, parent=None):
+    def __init__(self, names):
         self.names = names
-        super(Nonlocal, self).__init__(lineno, col_offset, parent)
 
     def _infer_name(self, frame, name):
         return name
@@ -1068,14 +828,6 @@ class Print(bases.Statement):
     dest = None
     values = None
 
-    def __init__(self, nl=None, lineno=None, col_offset=None, parent=None):
-        self.nl = nl
-        super(Print, self).__init__(lineno, col_offset, parent)
-
-    def postinit(self, dest=None, values=None):
-        self.dest = dest
-        self.values = values
-
 
 class Raise(bases.Statement):
     """class representing a Raise node"""
@@ -1084,20 +836,11 @@ class Raise(bases.Statement):
         _astroid_fields = ('exc', 'inst', 'tback')
         inst = None
         tback = None
-
-        def postinit(self, exc=None, inst=None, tback=None):
-            self.exc = exc
-            self.inst = inst
-            self.tback = tback
     else:
         _astroid_fields = ('exc', 'cause')
         exc = None
         cause = None
 
-        def postinit(self, exc=None, cause=None):
-            self.exc = exc
-            self.cause = cause
-
     def raises_not_implemented(self):
         if not self.exc:
             return
@@ -1111,9 +854,6 @@ class Return(bases.Statement):
     _astroid_fields = ('value',)
     value = None
 
-    def postinit(self, value=None):
-        self.value = value
-
 
 class Set(_BaseContainer):
     """class representing a Set node"""
@@ -1129,20 +869,11 @@ class Slice(bases.NodeNG):
     upper = None
     step = None
 
-    def postinit(self, lower=None, upper=None, step=None):
-        self.lower = lower
-        self.upper = upper
-        self.step = step
-
-
 class Starred(mixins.ParentAssignTypeMixin, bases.NodeNG):
     """class representing a Starred node"""
     _astroid_fields = ('value',)
     value = None
 
-    def postinit(self, value=None):
-        self.value = value
-
 
 class Subscript(bases.NodeNG):
     """class representing a Subscript node"""
@@ -1150,10 +881,6 @@ class Subscript(bases.NodeNG):
     value = None
     slice = None
 
-    def postinit(self, value=None, slice=None):
-        self.value = value
-        self.slice = slice
-
 
 class TryExcept(mixins.BlockRangeMixIn, bases.Statement):
     """class representing a TryExcept node"""
@@ -1162,11 +889,6 @@ class TryExcept(mixins.BlockRangeMixIn, bases.Statement):
     handlers = None
     orelse = None
 
-    def postinit(self, body=None, handlers=None, orelse=None):
-        self.body = body
-        self.handlers = handlers
-        self.orelse = orelse
-
     def _infer_name(self, frame, name):
         return name
 
@@ -1189,10 +911,6 @@ class TryFinally(mixins.BlockRangeMixIn, bases.Statement):
     body = None
     finalbody = None
 
-    def postinit(self, body=None, finalbody=None):
-        self.body = body
-        self.finalbody = finalbody
-
     def block_range(self, lineno):
         """handle block line numbers range for try/finally statements"""
         child = self.body[0]
@@ -1210,39 +928,14 @@ class Tuple(_BaseContainer):
         return '%s.tuple' % BUILTINS
 
     def getitem(self, index, context=None):
-        return _container_getitem(self, self.elts, index)
+        return self.elts[index]
 
 
 class UnaryOp(bases.NodeNG):
     """class representing an UnaryOp node"""
     _astroid_fields = ('operand',)
-    _other_fields = ('op',)
     operand = None
 
-    def __init__(self, op=None, lineno=None, col_offset=None, parent=None):
-        self.op = op
-        super(UnaryOp, self).__init__(lineno, col_offset, parent)
-
-    def postinit(self, operand=None):
-        self.operand = operand
-
-    # This is set by inference.py
-    def _infer_unaryop(self, context=None):
-        raise NotImplementedError
-
-    def type_errors(self, context=None):
-        """Return a list of TypeErrors which can occur during inference.
-
-        Each TypeError is represented by a :class:`UnaryOperationError`,
-        which holds the original exception.
-        """
-        try:
-            results = self._infer_unaryop(context=context)
-            return [result for result in results
-                    if isinstance(result, exceptions.UnaryOperationError)]
-        except exceptions.InferenceError:
-            return []
-
 
 class While(mixins.BlockRangeMixIn, bases.Statement):
     """class representing a While node"""
@@ -1251,11 +944,6 @@ class While(mixins.BlockRangeMixIn, bases.Statement):
     body = None
     orelse = None
 
-    def postinit(self, test=None, body=None, orelse=None):
-        self.test = test
-        self.body = body
-        self.orelse = orelse
-
     @decorators.cachedproperty
     def blockstart_tolineno(self):
         return self.test.tolineno
@@ -1271,10 +959,6 @@ class With(mixins.BlockRangeMixIn, mixins.AssignTypeMixin, bases.Statement):
     items = None
     body = None
 
-    def postinit(self, items=None, body=None):
-        self.items = items
-        self.body = body
-
     @decorators.cachedproperty
     def blockstart_tolineno(self):
         return self.items[-1][0].tolineno
@@ -1297,14 +981,14 @@ class Yield(bases.NodeNG):
     _astroid_fields = ('value',)
     value = None
 
-    def postinit(self, value=None):
-        self.value = value
-
-
 class YieldFrom(Yield):
     """ Class representing a YieldFrom node. """
 
 
+class DictUnpack(bases.NodeNG):
+    """Represents the unpacking of dicts into dicts using PEP 448."""
+
+
 # constants ##############################################################
 
 CONST_CLS = {
@@ -1344,7 +1028,7 @@ def const_factory(value):
 
 
 # Backward-compatibility aliases
-def _instancecheck(cls, other):
+def instancecheck(cls, other):
     wrapped = cls.__wrapped__
     other_cls = other.__class__
     is_instance_of = wrapped is other_cls or issubclass(other_cls, wrapped)
@@ -1358,7 +1042,7 @@ def _instancecheck(cls, other):
 def proxy_alias(alias_name, node_type):
     proxy = type(alias_name, (lazy_object_proxy.Proxy,),
                  {'__class__': object.__dict__['__class__'],
-                  '__instancecheck__': _instancecheck})
+                  '__instancecheck__': instancecheck})
     return proxy(lambda: node_type)
 
 Backquote = proxy_alias('Backquote', Repr)
